/*
 * lcd.c
 *
 * Created: 18.05.2012 11:25:13
 *  Author: Marko Jankovec, Matija Pirc
 */ 
#include "lcd.h"

FILE LCDstream=FDEV_SETUP_STREAM(LCDputc, NULL, _FDEV_SETUP_WRITE);

unsigned char x=0, y=0;

int LCDputc(char c, FILE* stream)
{
	switch (c)
	{
		case '\n': LCD_GotoXY(1-y,x); break;
		case '\r': LCD_GotoXY(y,0); break;
		case '\b': LCD_GotoXY(y,x-1); LCD_Data(' '); LCD_GotoXY(y,x-1); break;
		case '\t': if (x < 12) LCD_GotoXY(y,((x+1)>>2)<<2); else LCD_GotoXY(1-y,0); break;
		default:
			switch (c)
			{
				case 0xfd: c=0; break;
				case 0xfe: c=1; break;
				default: break;
			}				
			LCD_Data(c);
			x++;
			if (x>15) LCD_GotoXY(1-y,0);
			break;
	}
	return 0;
}

#define LCD_E_set   PORTC|=0x01;
#define LCD_E_clr   PORTC&=~0x01;
#define LCD_R		PORTC|=0x40;
#define LCD_W		PORTC&=~0x40;
#define LCD_DR      PORTC|=0x80;
#define LCD_IR      PORTC&=~0x80;

void LCD_Init()
{
	DDRC|=0xC1;  //set RS,RW and E as output
	DDRA |= 0xf0; //nastavi ustrezne pine kot izhode
	PORTD|=0x80; //prizgi osvetlitev
	_delay_ms(50);
       
	LCD_W
	LCD_IR
       
	LCD_E_set
	PORTA &= 0x0f;      // 0000 xxxx
	PORTA|=0x30; // 0011 0000
	LCD_E_clr
       
	_delay_ms(10);
       
	LCD_E_set
	PORTA &= 0x0f;      // 0000 xxxx
	PORTA|=0x30; // 0011 0000
	LCD_E_clr
       
	_delay_us(200);

	LCD_E_set
	PORTA &= 0x0f;      // 0000 xxxx
	PORTA|=0x30; // 0011 0000
	LCD_E_clr
       
	_delay_us(200);
       
	LCD_E_set
	PORTA &= 0x0f;      // 0000 xxxx
	PORTA|=0x20; // 0010 0000
	LCD_E_clr
       
	LCD_Cmd(0x28);      //Set display, 2 lines, 4-bit DL
	LCD_Cmd(0x08);      //Display off
	LCD_Cmd(0x01);      //LCD clear
	LCD_Cmd(0x06);      //Entry mode - auto increment, no shift
	LCD_Cmd(0x02);      //Go home
	LCD_Cmd(0x0F);      //Display on, cursor ON, blinking
}

void LCD_loadCustomChars(char adr,char *chr)
{
	int i;
	LCD_Cmd(0x40 | ((adr&0x07)<<3));
	for (i=0; i<8; i++)
	{
		LCD_Data(chr[i]);
	}
	LCD_GotoXY(x,y);
}

char LCD_Busy()
{
	char data;

	DDRA &= ~0xf0; //ker bomo brali nastavi ustrezne pine kot vhode
	//D4 - D7 = PORTA 4-7
	//RS(PC7) = 0, RW(PC6) = 1, E(PC0)
	LCD_IR
	LCD_R
       
	//Tu beremo zgornje 4 bite
	LCD_E_set
	//PWmin = 450ns = 8.3 cycles, 1cycle=1/18432000=54ns
	_delay_loop_1(3);	//3 cycles per loop * 3 =9cycles
	data = PINA; //preberi zgornje stiri bite
	LCD_E_clr
	_delay_loop_1(3);
       
	//Tu beremo spodnje 4 bite (samo da jih v bistvu ignoriramo)
	LCD_E_set
	_delay_loop_1(3);
	LCD_E_clr
       
	DDRA |= 0xf0; //nazaj nastavi pine kot izhode
	if (data & 0x80)    return 1;    // pomeni da je zaseden
	else return 0;      // pomeni da ni zaseden
}

void LCD_Cmd(char data)
{
	char temp;   
       
	while (LCD_Busy());
	//_delay_ms(2);
	//D4 - D7 = PORTA 4-7

	LCD_IR // pii v instrukcijski register
	LCD_W
	LCD_E_set
	// write high nibble

	PORTA &= 0x0f;      // 0000 xxxx
	temp = data & 0xf0; // dddd 0000
	PORTA |= temp;      // dddd xxxx

	LCD_E_clr
       
	_delay_loop_1(3);	//3 cycles per loop * 3 =9cycles
       
	LCD_E_set
       
	// write low nibble
	PORTA &= 0x0f;      // 0000 xxxx
	temp = data & 0x0f; //  0000 dddd
	temp <<= 4;         //  dddd 0000
	PORTA |= temp;      //     dddd xxxx
       
	LCD_E_clr
}

void LCD_Data(char data)
{
	char temp;   
       
	while (LCD_Busy());
	//_delay_ms(2);
	//D4 - D7 = PORTA 4-7

	LCD_DR // pii v podatkovni register
	LCD_W
	LCD_E_set
	// write high nibble

	PORTA &= 0x0f;      // 0000 xxxx
	temp = data & 0xf0; // dddd 0000
	PORTA |= temp;      // dddd xxxx

	LCD_E_clr
       
	_delay_loop_1(3);	//3 cycles per loop * 3 =9cycles
       
	LCD_E_set
       
	// write low nibble
	PORTA &= 0x0f;      // 0000 xxxx
	temp = data & 0x0f; //  0000 dddd
	temp <<= 4;         //  dddd 0000
	PORTA |= temp;      //     dddd xxxx
       
	LCD_E_clr
}

void LCD_Clear()
{
	x=0;
	y=0;
	LCD_Cmd(0x01);             
}

void LCD_Home()
{
	x=0;
	y=0;
    LCD_Cmd(0x02);
}

void LCD_Shift(char data)
{
	LCD_Cmd(0x10|(data&0x0F));
}

void LCD_GotoXY(char row, char coloumn)
{
	if (row<2 && coloumn<16)
	{
		y=row;
		x=coloumn;
		LCD_Cmd(0x80+coloumn+0x40*row);   
	}
}

void LCD_Control(char data)
{
	LCD_Cmd(0x08|(data&0x0F));
}