/*
 * menus.c
 *
 * Created: 12.06.2012 12:19:46
 *  Author: maticpi
 */ 

#include "menus.h"

void SettingsMenu()
{
	char c;
	do
	{
		fprintf_P(&RS232str,PSTR("\r\nSettings - main menu:\r\n"));
		fprintf_P(&RS232str,PSTR("   1 - Set Alarm thresholds (Min:%d%%, Max:%d%%, Hyst:%d%%)\r\n"),LowAlarmLevel,HighAlarmLevel,Hysteresis);
		fprintf_P(&RS232str,PSTR("   2 - Set minimum and maximum sensor values  (Min:%d, Max:%d)\r\n"),bEmpty,bFull);
		fprintf_P(&RS232str,PSTR("   s - Save parameters to EEPROM\r\n"),Hysteresis);
		fprintf_P(&RS232str,PSTR("   ESC or x - Exit menu\r\n"));
		while (!(UCSR0A & (1<<RXC0))) ;	//Wait for new command
		c=UDR0;
		switch (c)
		{
			case '1': AlarmTresholdMenu(); break;
			case '2': MeasurementRangeMenu(); break;
			case 's': SavetoEEPROM(); fprintf_P(&RS232str,PSTR("\r\nSettings saved!\r\n")); break;
			case 'x':	//exit menu command - nothing to do here
			case 27:  break;
			default:
			fprintf_P(&RS232str,PSTR("Unknown selection - try again!\r\n"));
			break;
		}
	} while (c != 27 /*ESC*/ && c !='x');
}

void MeasurementRangeMenu()
{
	char c;
	int val;
	do
	{
		fprintf_P(&RS232str,PSTR("\r\nSettings - measurement range menu:\r\n"));
		fprintf_P(&RS232str,PSTR("   1 - Barrel empty AD value (current value %d)\r\n"),  bEmpty);
		fprintf_P(&RS232str,PSTR("   2 - Barrel full AD value (current value %d)\r\n"),  bFull);
		fprintf_P(&RS232str,PSTR("   ESC or x - back to main menu\r\n"));
		while (!(UCSR0A & (1<<RXC0))) ;	//Wait for new command
		c=UDR0;
		switch (c)
		{
			case '1':
			fprintf_P(&RS232str,PSTR("\r\nSettings - Enter new barrel empty AD value (0 - %d):"),bFull-1);
			val=ReadRS232value();
			if (val <0) fprintf_P(&RS232str,PSTR(" - canceled!\r\n"));
			else if (val >= bFull) fprintf_P(&RS232str,PSTR(" - invalid entry. Minimum value can't be higher than or equal to the maximum value.\r\n"));
			else
			{
				bEmpty=val;
				bSpan=bFull-bEmpty;
			}
			break;
			case '2':
			fprintf_P(&RS232str,PSTR("\r\nSettings - Enter new barrel full AD level (%d - 1023):"),bEmpty+1);
			val=ReadRS232value();
			if (val <0) fprintf_P(&RS232str,PSTR(" - canceled!\r\n"));
			else if (val > 1023) fprintf_P(&RS232str,PSTR(" - invalid entry. Maximum value can't exceed 1023.\r\n"));
			else if (val <= bEmpty) fprintf_P(&RS232str,PSTR(" - invalid entry. Maximum value can't be lower than or equal to the minimum value.\r\n"));
			else
			{
				bFull=val;
				bSpan=bFull-bEmpty;
			}
			break;
			case 'x':	//exit menu command - nothing to do here
			case 27:
			break;
			default:
			fprintf_P(&RS232str,PSTR("Unknown selection - try again!\r\n"));
			break;
		}
	} while (c != 27 /*ESC*/ && c !='x');
}

void AlarmTresholdMenu()
{
	char c;
	int val;
	do
	{
		fprintf_P(&RS232str,PSTR("\r\nSettings - alarm threshold menu:\r\n"));
		fprintf_P(&RS232str,PSTR("   1 - Barrel empty alarm threshold (current value %d%%)\r\n"),LowAlarmLevel);
		fprintf_P(&RS232str,PSTR("   2 - Barrel full alarm threshold  (current value %d%%)\r\n"),HighAlarmLevel);
		fprintf_P(&RS232str,PSTR("   3 - Threshold Hysteresis (current value %d%%)\r\n"),Hysteresis);
		fprintf_P(&RS232str,PSTR("   ESC or x - Exit menu\r\n"));
		while (!(UCSR0A & (1<<RXC0))) ;	//Wait for new command
		c=UDR0;
		switch (c)
		{
			case '1':
			fprintf_P(&RS232str,PSTR("\r\nSettings - Enter new barrel empty alarm level (0 - %d):"),HighAlarmLevel-Hysteresis);
			val=ReadRS232value();
			if (val <0) fprintf_P(&RS232str,PSTR(" - canceled!\r\n"));
			else if (val > HighAlarmLevel-Hysteresis) fprintf_P(&RS232str,PSTR(" - invalid entry. Empty alarm level must be less than Full alarm level (with hysteresis).\r\n"));
			else LowAlarmLevel=val;
			break;
			case '2':
			fprintf_P(&RS232str,PSTR("\r\nSettings - Enter new barrel full alarm level (%d - 100):"),LowAlarmLevel+Hysteresis);
			val=ReadRS232value();
			if (val <0) fprintf_P(&RS232str,PSTR(" - canceled!\r\n"));
			else if (val > 100) fprintf_P(&RS232str,PSTR(" - invalid entry. Full alarm level can't exceed 100%%.\r\n"));
			else if (val < LowAlarmLevel+Hysteresis) fprintf_P(&RS232str,PSTR(" - invalid entry. Full alarm level must be more than Empty alarm level (with hysteresis).\r\n"));
			else HighAlarmLevel=val;
			break;
			case '3':
			fprintf_P(&RS232str,PSTR("\r\nSettings - Enter hysteresis for alarm levels (0 - %d):"),HighAlarmLevel-LowAlarmLevel);
			val=ReadRS232value();
			if (val <0) fprintf_P(&RS232str,PSTR(" - canceled!\r\n"));
			else if (val > HighAlarmLevel-LowAlarmLevel) fprintf_P(&RS232str,PSTR(" - invalid entry. Hysteresis can't exceed the difference between Full and Empty alarm levels.\r\n"));
			else Hysteresis=val;
			break;
			case 'x':	//exit menu command - nothing to do here
			case 27:
			break;
			default:
			fprintf_P(&RS232str,PSTR("Unknown selection - try again!\r\n"));
			break;
		}
	} while (c != 27 /*ESC*/ && c !='x');
}
